
import java.net.*;
import java.io.*;
import java.lang.*;
import java.util.*;

/** Name: Connection
 * Desc: This object is instantiated when a Server object accepts an incomming
 * connection from a nsjc client. When that happens this object attempts to 
 * retrieve the test information being sent from the nsjc client. The information
 * comes over in three data packets, retrieved by three readLine method calls into
 * a Buffered Reader object. When the three lines are received then the connection
 * is closed, the results are written to the command file, and the thread ends
 *
 * Note that the Connection object won't start processing the information from the 
 * nsjc client until it is started. Because this object extends Thread, you have
 * to call the 'run' method to start it's reading of the data.
 *
 * @author viaggi@nietzsche.net
 */
public class Connection extends Thread{
        private Socket client;
        private DataInputStream disInput;
        private BufferedReader brInput;

        private PrintStream psOutput;
        
        private String ivar_hostname;
        private nsjsd ivar_parent;
        
        /** Name: Connection
         * Desc: This function is the constructor and the only way, currently,
         * that you can initialize the internal instance variables to run the 
         * object. This object accepts the socket that the client is connecting
         * on, the nsjsd parent, and the name of the host to add to the string
         * written to the command file. The reason the host is being passed in
         * is in-case you have an alias setup, in which case attempting to identify
         * the host by way of the Socket will give the wrong host info.
         * 
         * @param client_socket --> Socket, the socket the client is connecting on.
         * @param nsjParent --> the nsjsd parent object
         * @param strHost --> String, the name of the host to write to the command file
         */
        public Connection(Socket client_socket, nsjsd nsjParent, String strHost){
            
            client = client_socket;
            ivar_parent = nsjParent;
            ivar_hostname = strHost;
            
            try
            {
                //Initialize the Input Stream, Print Stream and the Buffered Reader
                disInput = new DataInputStream(client.getInputStream());
                brInput  = new BufferedReader(new InputStreamReader(disInput));
                psOutput = new PrintStream(client.getOutputStream());
            }
            catch (IOException e)
            {
                try 
                {client.close();}
                catch (IOException e2)
                {;}
                ivar_parent.writelog("Exception while getting socket streams: " + e.toString());
                return;
            }
            
        }
        
        /** Name: run
         * Desc: This function is provided because this object implements the runnable
         * interface. This means that this function is called when the thread is started
         * as a separate process. It essentially collects the information that the client
         * is sending to this server and turns the stream into a string that will 
         * report the status to the command file. 
         */
        public void run(){
            
                String strLine;
                StringBuffer stbCollect;

                try
                {
                    //Start building the command string
                    stbCollect = new StringBuffer("PROCESS_SERVICE_CHECK_RESULT;" + ivar_hostname + ";");
                    
                    //Loop forever
                    for(;;)
                    {
                        
                        //Read the first line from the Buffered Reader
                        //Should be the Test Name
                        strLine = brInput.readLine();
                        
                        //If the line is not null then collect the data
                        //If the line is null then, break from the loop
                        if (strLine != null) 
                        {
                            stbCollect.append(strLine + ";");
                            
                            //Collect the Test results
                            strLine = brInput.readLine();
                            stbCollect.append(strLine + ";");
                            
                            //Collect the Test Comments
                            strLine = brInput.readLine();
                            stbCollect.append(strLine);
                            
                            //Build the command String and write it to the command file
                            strLine = new String(stbCollect);
                            ivar_parent.writeCommand(strLine);
                        
                            //Print to the output stream the command string just written
                            psOutput.println(strLine);
                            
                        }
                        else {break;}
                    }
                    
                }
                catch (IOException e)
                {;}
                finally 
                {try 
                    {client.close();}
                 catch(IOException e2)
                    {;}
                }
        }
  }


